<?php
class Quizz extends Tools_Plugins_Abstract {

    const CONDITION_BAD = 'bad';
    const CONDITION_GOOD = 'good';
    const CONDITION_BEST = 'best';

    /**
     * Zend layout instance to render all the plugin's screens
     *
     * @var Zend_Layout
     */
    protected $_layout       = null;

    /**
     * @var Helpers_Action_Config
     */
    private $_configHelper;

    /**
     * init widget
     */
    protected function _init() {
        $this->_view->addScriptPath(__DIR__ . '/system/app/Widgets/Quizz/views/');
        // initialize helpers
        $this->_configHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('config');
    }

    /**
     * Delete quiz
     */
    public function deleteQuizAction(){
        if($this->_request->isPost() && Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)){
            $pageId = $this->_request->getParam('pageId');
            $quizName = filter_var($this->_request->getParam('quizName'), FILTER_SANITIZE_STRING);
            Quizz_Models_Mappers_QuizMapper::getInstance()->deleteQuiz($quizName, $pageId);
            $this->_responseHelper->success($this->_translator->translate('Deleted'));
        }
    }

    /**
     * Save quiz data
     */
    public function saveQuizDataAction(){
        if($this->_request->isPost() && Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)){
            $quizTitle = filter_var($this->_request->getParam('quizTitle'), FILTER_SANITIZE_STRING);
            $quizName = filter_var($this->_request->getParam('quizName'), FILTER_SANITIZE_STRING);
            $pageId = filter_var($this->_request->getParam('pageId'), FILTER_SANITIZE_NUMBER_INT);
            $conditions = $this->_request->getParam('conditions');
            $quizData = $this->_request->getParam('quizData');
            $quizMapper = Quizz_Models_Mappers_QuizMapper::getInstance();
            $quizModel = new Quizz_Models_Model_Quiz();
            $quizModel->setPageId($pageId);
            $quizModel->setQuizName($quizName);
            $quizModel->setQuizTitle($quizTitle);
            $quizModel->setPublished(1);
            $quizModel->setModifyDate(date(DATE_ATOM));
            $quizExist = $quizMapper->checkExistingQuiz($quizName, $pageId);
            if(empty($quizExist)){
                $quizId = $quizMapper->save($quizModel);
            }else{
                $quizId = $quizExist[0]->getId();
            }
            if(is_array($conditions)) {
                $quizMapper->saveConditions($quizId, $conditions);
            }elseif(is_array($quizData)){
                //process quizData
                foreach($quizData as $key => $quizInfo){
                    $quizModel->setQuestions(
                        $key,
                        trim($quizData[$key]['question']),
                        trim(filter_var($quizData[$key]['point'],FILTER_SANITIZE_NUMBER_INT)),
                        intval($quizData[$key]['published'])
                    );
                    $quizModel->setAnswers($quizData[$key]['answers']);
                }
            }
            $quizMapper->save($quizModel);
            $this->_responseHelper->success($quizId);
        }

    }

    public function deleteQuestionAction(){
        if($this->_request->isPost() && Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)){
            $quizName = filter_var($this->_request->getParam('quizName'), FILTER_SANITIZE_STRING);
            $pageId = filter_var($this->_request->getParam('pageId'), FILTER_SANITIZE_NUMBER_INT);
            $questionPosition = filter_var($this->_request->getParam('questionId'), FILTER_SANITIZE_NUMBER_INT);
            $quizMapper = Quizz_Models_Mappers_QuizMapper::getInstance();
            $quizExist = $quizMapper->checkExistingQuiz($quizName, $pageId);
            if(!empty($quizExist)){
                $quizId = $quizExist[0]->getId();
                $quizMapper->deleteQuestion($quizId, $questionPosition);
            }
            $this->_responseHelper->success('');
        }
    }


    /**
     * Return next part of questions
     */
    public function nextQuestionsAction(){
        $quizId = filter_var($this->_request->getParam('quizId'), FILTER_SANITIZE_NUMBER_INT);
        $perPage = filter_var($this->_request->getParam('perPage'), FILTER_SANITIZE_NUMBER_INT);
        $pageId = filter_var($this->_request->getParam('pageId'), FILTER_SANITIZE_NUMBER_INT);
        $quizOffset = filter_var($this->_request->getParam('quizOffset'), FILTER_SANITIZE_NUMBER_INT);
        $answers = $this->_request->getParam('answers');
        $currentQuizSessionKey = 'currentQuizQuestions'.$quizId;
        $currentQuizAnswersKey = 'currentQuizAnswers'.$quizId;

        if($this->_request->isPost() && isset($this->_sessionHelper->$currentQuizSessionKey) && !empty($answers)) {
            $offset = $quizOffset+$perPage;
            $currentQuiz = $this->_sessionHelper->$currentQuizSessionKey;
            if(isset($this->_sessionHelper->$currentQuizAnswersKey)){
                $answeredQuestions = $this->_sessionHelper->$currentQuizAnswersKey;
                foreach($answers as $key => $answer){
                    $answeredQuestions[$key] = $answer;
                }
                $this->_sessionHelper->$currentQuizAnswersKey = $answeredQuestions;
            }else{
                $this->_sessionHelper->$currentQuizAnswersKey = $answers;
            }
            $this->_view->quistionNumberStart = $offset;
            $quizQuestions = array_slice($currentQuiz, $offset, $perPage);

            if (empty($quizQuestions)) {
                // Form to save user info
                $form = new Quizz_Forms_SaveUserInfo();
                $form->setAnswersKey($currentQuizAnswersKey)->setQuizzId($quizId);
                $contactFormMapper = Quizz_Models_Mappers_CFMapper::getInstance();
                $contactFormData = unserialize($contactFormMapper->getFormDataByQuizID($quizId));
                $this->_view->formOrderElements = $form->renderForm($quizId, $contactFormData);
                $this->_view->form       = $form;
                $this->_view->websiteUrl = $this->_websiteHelper->getUrl();
                $this->_view->quizId = $quizId;
                $this->_view->pageId = $pageId;
                $view = $this->_view->render('form.save.user.info.phtml');
                $page = Application_Model_Mappers_PageMapper::getInstance()->findByUrl('index.html');
                $contentParser = new Tools_Content_Parser($view, $page->toArray(), array('websiteUrl' => $this->_websiteHelper->getUrl()));
                $view = $contentParser->parseSimple();
                $this->_responseHelper->success(
                    array(
                        'quizQuestions' => $view,
                        'noQuestions'   => true
                    )
                );
            }
            $this->_view->quizQuestions = $quizQuestions;
            $questionsView = $this->_view->render('quiz-questions.phtml');
            $this->_responseHelper->success(array('quizQuestions' => $questionsView, 'quizOffset' => $offset, 'noQuestions' => false));
        }
    }

    /**
     * Save user info and show results
     */
    public function showResultsAction() {
        if ($this->_request->isPost()) {
            $form = new Quizz_Forms_SaveUserInfo();
            $data = $this->_request->getParams();

            // Show results
            $quizKey    = 'currentQuiz'.$data['quizzId'];
            $answersKey = $data['answersKey'];

            if (isset($this->_sessionHelper->$answersKey) && isset($this->_sessionHelper->$quizKey)) {

                $answeredQuestions    = $this->_sessionHelper->$answersKey;
                $allQuizData          = $this->_sessionHelper->$quizKey;
                $quizMapper           = Quizz_Models_Mappers_QuizMapper::getInstance();
                $corrAnswers          = $quizMapper->getCorrectAnswers($data['quizzId']);
                $correctQuesitionsIds = array();

                foreach ($answeredQuestions as $keyAnswer => $answer) {
                    if ($corrAnswers[$keyAnswer]['weight'] == $answer) {
                        $correctQuesitionsIds[$keyAnswer] = $keyAnswer;
                    }
                }

                $score = 0;
                if (!empty($correctQuesitionsIds)) {
                    $score = $quizMapper->calculateQuizResult($correctQuesitionsIds);
                }

                $quizConditions = $quizMapper->getQuizConditions($data['quizzId']);
                $condition      = self::CONDITION_BAD;
                //get content from container for condition

                $contentMapper = Application_Model_Mappers_ContainerMapper::getInstance();

                if ($quizConditions[self::CONDITION_BAD]['points'] >= $score){
                    $condition = self::CONDITION_BAD;

                }
                elseif ($quizConditions[self::CONDITION_BAD]['points'] <= $score
                    && $quizConditions[self::CONDITION_GOOD]['points'] >= $score
                ) {
                    $condition = self::CONDITION_GOOD;
                }
                elseif ($quizConditions[self::CONDITION_BEST]['points'] >= $score) {
                    $condition = self::CONDITION_BEST;
                }
                // Save user result
                $scorePercent = 0;
                if (!empty($correctQuesitionsIds)) {
                    $scorePercent = sizeof($correctQuesitionsIds)/sizeof($answeredQuestions)*100;
                }

                $containerConditionalName = $allQuizData['quizName'].$condition;
                $quizContainer = $contentMapper->findByName($containerConditionalName, 0,  Application_Model_Models_Container::TYPE_STATICCONTENT);

                $quizScoreKey = 'score'.$data['quizzId'];
                $this->_sessionHelper->$quizScoreKey = $score;
                $quizScoreKey = 'percent'.$data['quizzId'];
                $this->_sessionHelper->$quizScoreKey = $scorePercent;

                $quizFormId                       = md5(microtime());
                $this->_view->quizName            = $allQuizData['quizName'];
                $this->_view->condition           = $condition;
                $this->_sessionHelper->quizFormId = $quizFormId;
                $this->_view->quizFormId          = $quizFormId;
                $this->_view->title               = $allQuizData['quizTitle'];
                $this->_view->score               = $score;
                $this->_view->scorePercent        = $scorePercent;
                $view                             = $this->_view->render('result.quiz.phtml');

                if (!$form->isValid($data)) {
                    $this->_responseHelper->fail($this->_translator->translate('Invalid name or email'));
                }

                // Save user result
                $scorePercent = 0;
                if (!empty($correctQuesitionsIds)) {
                    $scorePercent = sizeof($correctQuesitionsIds)/sizeof($answeredQuestions)*100;
                }

                $userDbTable = new Quizz_Models_DbTables_UserDbTable();
                $userDbTable->insert(
                    array(
                        'firstname'        => $data['firstName'],
                        'lastname'         => $data['lastName'],
                        'phone'            => $data['phone'],
                        'email'            => $data['email'],
                        'created_at'       => date(DATE_ATOM),
                        'score'            => $score,
                        'score_percent'    => $scorePercent,
                    )
                );
                $page          = Application_Model_Mappers_PageMapper::getInstance()->findByUrl('index.html');
                $quiz = $quizMapper->find($allQuizData['quizId']);
                $phone = (isset($data['phone'])) ? $data['phone'] : '';
                $contentParserQuiz = new Tools_Content_Parser(
                    $quizContainer->getContent(),
                    $page->toArray(),
                    array('websiteUrl' => $this->_websiteHelper->getUrl())
                );

                $quiz->registerObserver(new Tools_Mail_Watchdog(array(
                        'trigger'          => Quizz_Tools_MailWatchdog::TRIGGER_QUIZ_PASSED,
                        'quizId'           => $allQuizData['quizId'],
                        'mailTo'           => $data['email'],
                        'score'            => $score,
                        'title'            => $allQuizData['quizTitle'],
                        'userName'         => $data['firstName'].''.$data['lastName'],
                        'phone'            => $phone,
                        'date'             => date('D jS M Y'),
                        'custom_mail_body' =>  $contentParserQuiz->parseSimple(),
                    )));
                $quiz->notifyObservers($quiz);

                $contentParser = new Tools_Content_Parser(
                    $view,
                    $page->toArray(),
                    array('websiteUrl' => $this->_websiteHelper->getUrl())
                );
                $view          = $contentParser->parseSimple();

                $this->_responseHelper->success(array('quizQuestions' => $view, 'noQuestions' => true));
            }
        }
    }

    /**
     * Export file csv
     */
    public function exportCsvAction() {
        if (!Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)) {
            $this->_redirector->gotoUrl(Zend_Controller_Action_HelperBroker::getStaticHelper('website')->getUrl());
        }

        $header      = array('firstname', 'lastname', 'phone', 'email', 'created', 'score', 'score, %');
        $columns     = array('firstname', 'lastname', 'phone', 'email', 'created_at', 'score', 'score_percent');
        $userDbTable = new Quizz_Models_DbTables_UserDbTable();
        $select      = $userDbTable->select()->from($userDbTable, $columns);
        $exportData  = array_merge(array($header), $userDbTable->fetchAll($select)->toArray());
        $fileName    = Widgets_Quizzexport_Quizzexport::FILE_PREFIX.Widgets_Quizzexport_Quizzexport::FILE_NAME;

        header('Content-Type: text/csv;charset=utf-8');
        header('Content-Disposition: attachment;filename="'.$fileName.'_'.date('Y_m_d').'.csv"');
        header('Pragma: no-cache');
        header('Expires: 0');

        $csv = fopen('php://output', Widgets_Quizzexport_Quizzexport::CSV_DELIMITER);
        foreach ($exportData as $line) {
            fputcsv($csv, $line);
        }
        fclose($csv);

        return $csv;
    }

    /**
     * Calculate quiz results and return email form
     * @deprecated
     */
    public function submitResultsAction(){
        $quizId = filter_var($this->_request->getParam('quizId'), FILTER_SANITIZE_NUMBER_INT);
        if($this->_request->isPost() && is_numeric($quizId)){
            $currentQuizAnswersKey = 'currentQuizAnswers'.$quizId;
            $currentQuizKey = 'currentQuiz'.$quizId;
            if(isset($this->_sessionHelper->$currentQuizAnswersKey) && isset($this->_sessionHelper->$currentQuizKey)){
                $answers = $this->_sessionHelper->$currentQuizAnswersKey;
                $allQuizData = $this->_sessionHelper->$currentQuizKey;
                $quizMapper = Quizz_Models_Mappers_QuizMapper::getInstance();
                $corrAnswers = $quizMapper->getCorrectAnswers($quizId);
                $correctQuesitionsIds = array();
                foreach($answers as $keyAnswer => $answer){
                    if($corrAnswers[$keyAnswer]['weight'] == $answer){
                        $correctQuesitionsIds[$keyAnswer] = $keyAnswer;
                    }
                }
                $score = 0;
                if(!empty($correctQuesitionsIds)){
                    $score = $quizMapper->calculateQuizResult($correctQuesitionsIds);
                }

                $quizConditions = $quizMapper->getQuizConditions($quizId);
                $condition = self::CONDITION_BAD;
                if($quizConditions[self::CONDITION_BAD]['points'] >= $score){
                    $condition = self::CONDITION_BAD;
                }elseif($quizConditions[self::CONDITION_BAD]['points'] <= $score && $quizConditions[self::CONDITION_GOOD]['points'] >= $score){
                    $condition = self::CONDITION_GOOD;
                }elseif($quizConditions[self::CONDITION_BEST]['points'] >= $score){
                    $condition = self::CONDITION_BEST;
                }
                //TODO change for something more proper
                $quizFormId = md5(microtime());

                $this->_view->condition = $condition;
                $this->_sessionHelper->quizFormId = $quizFormId;
                $this->_view->quizFormId = $quizFormId;
                $this->_view->title = $allQuizData['quizTitle'];
                $this->_view->score = $score;
                $view = $this->_view->render('result.quiz.phtml');
                $page = Application_Model_Mappers_PageMapper::getInstance()->findByUrl('index.html');
                $contentParser = new Tools_Content_Parser($view, $page->toArray(), array('websiteUrl' => $this->_websiteHelper->getUrl()));
                $view = $contentParser->parseSimple();
                $this->_responseHelper->success(array('html' => $view));
            }
        }
    }

    /**
     * @deprecated
     */
    public function sendResultAction(){
        $quizId = filter_var($this->_request->getParam('quizId'), FILTER_SANITIZE_NUMBER_INT);
        $userEmail = filter_var($this->_request->getParam('userEmail'), FILTER_SANITIZE_STRING);
        $quizScore = filter_var($this->_request->getParam('quizScore'), FILTER_SANITIZE_NUMBER_INT);
        $quizTitle = filter_var($this->_request->getParam('quizTitle'), FILTER_SANITIZE_STRING);
        $userName = filter_var($this->_request->getParam('userName'), FILTER_SANITIZE_STRING);
        $quizFormKey = $this->_request->getParam('quizFormKey');
        if($this->_request->isPost() && isset($quizId) && isset($this->_sessionHelper->quizFormId)){

            if($this->_sessionHelper->quizFormId !== $quizFormKey){
                $this->_responseHelper->fail('Send mail failed');
            }

            $validator = new Zend_Validate_EmailAddress();
            if (!$validator->isValid($userEmail)) {
                $this->_responseHelper->fail($this->_translator->translate('Email not valid'));
            }
            $quizMapper = Quizz_Models_Mappers_QuizMapper::getInstance();
            $quiz = $quizMapper->find($quizId);

            $quiz->registerObserver(new Tools_Mail_Watchdog(array(
                    'trigger' => Quizz_Tools_MailWatchdog::TRIGGER_QUIZ_PASSED,
                    'mailTo'  => $userEmail,
                    'score' => $quizScore,
                    'title' => $quizTitle,
                    'userName'  => $userName
                )));
            $quiz->notifyObservers($quiz);
            return $this->_responseHelper->success($this->_translator->translate('Email send'));
        }
    }

    /**
     * Save quiz data
     * @deprecated
     */
    public function saveQuizAction(){
        if($this->_request->isPost() && Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)){
            $data = $this->_request->getParams();
            $quizCondition = filter_var_array($data['conditions'], FILTER_SANITIZE_NUMBER_INT);
            $quizTitle = filter_var($data['quizTitle'], FILTER_SANITIZE_STRING);
            $quizName = filter_var($data['quizName'], FILTER_SANITIZE_STRING);
            $pageId = filter_var($data['pageId'], FILTER_SANITIZE_NUMBER_INT);
            $quizData = $data['quizData'];
            $quizMapper = Quizz_Models_Mappers_QuizMapper::getInstance();
            $quizModel = new Quizz_Models_Model_Quiz();

            $quizModel->setPageId($pageId);
            $quizModel->setQuizName($quizName);
            $quizModel->setQuizTitle($quizTitle);
            $quizModel->setPublished(1);
            $quizModel->setModifyDate(date(DATE_ATOM));

            //process quizData
            foreach($quizData as $key => $quizInfo){
                $quizModel->setQuestions(
                    $key,
                    trim(filter_var($quizData[$key]['question'], FILTER_SANITIZE_STRING)),
                    trim(filter_var($quizData[$key]['point'],FILTER_SANITIZE_NUMBER_INT))
                );
                $quizModel->setAnswers($key, $quizData[$key]['answers']);
            }

            if(!empty($quizCondition)){
                $quizModel->setConditions($quizCondition);
            }

            $quizMapper->save($quizModel);
            $this->_responseHelper->success($this->_translator->translate('Saved'));
        }
    }

    /**
     * Save quiz contact form coniguration
     */
    public function saveFormConfAction(){

        if($this->_request->isXmlHttpRequest() && Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_CONTENT)){
            $quizId   = filter_var($this->_request->getParam('quizId'), FILTER_SANITIZE_NUMBER_INT);
            $parseFormData = $this->_request->getParam('formData');
            $parseFormData['firstName'] = empty($parseFormData['firstName']) ? $this->_translator->translate('First Name') : $parseFormData['firstName'];
            $parseFormData['lastName'] = empty($parseFormData['lastName']) ? $this->_translator->translate('Last Name') : $parseFormData['lastName'];
            //email field is always required and publised in contact form
            $parseFormData['email'] = empty($parseFormData['email']) ? $this->_translator->translate('Email') : $parseFormData['email'];
            $parseFormData['is_published_email'] = $parseFormData['is_required_email'] = true;
            $parseFormData['phone'] = empty($parseFormData['phone']) ? $this->_translator->translate('Phone (optional)') : $parseFormData['phone'];
            //set order for form fields
            $defaultKeys = array_fill_keys(Quizz_Forms_SaveUserInfo::defaultFormElements(), '');
            $parseFormData['order']  = array_intersect_key($parseFormData,$defaultKeys);
            //end
            $cfsettingsMapper = Quizz_Models_Mappers_CFMapper::getInstance();
            $cfsettingsModel  = new Quizz_Models_Model_CFModel();
            $cfsettingsModel->setData(serialize($parseFormData));
            $cfsettingsModel->setQuizId($quizId);
            $cfsettingsMapper -> save($cfsettingsModel);
            $this->_responseHelper->success($this->_translator->translate('Configuration saved'));
        }

    }

}