<?php
class Quizz_Forms_SaveUserInfo extends Zend_Form {

    protected $_answersKey;

    protected $_quizzId;

    public function getAnswersKey() {
        return $this->_answersKey;
    }

    public function setAnswersKey($_answersKey) {
        $this->_answersKey = $_answersKey;
        $this->getElement('answersKey')->setValue($this->_answersKey);
        return $this;
    }

    public function getQuizzId() {
        return $this->_quizzId;
    }

    public function setQuizzId($_quizzId) {
        $this->_quizzId = $_quizzId;
        $this->getElement('quizzId')->setValue($this->_quizzId);
        return $this;
    }

    /**
     * Set admin's configuratons for contact form
     *
     * @param int $quizId
     *
     * @return $this
     */
    public function renderForm($quizId, $contactFormData){
        $defaultElements = self::defaultFormElements();
        foreach($defaultElements as $defaultElement){
            if(empty($contactFormData['is_published_'.$defaultElement])){
                $this->removeElement($defaultElement);
                unset($contactFormData['order'][$defaultElement]);
            } else {
                $this->getElement($defaultElement)->setAttribs(array('placeholder' => $contactFormData[$defaultElement]));
                if(!empty($contactFormData['is_required_'.$defaultElement])){
                    $this->getElement($defaultElement)->setRequired(true);
                    $this->getElement($defaultElement)->setAttrib('class','required');
                }
            }
        }
        return $contactFormData['order'];
    }

    /**
     *
     */
    public function init() {
        $this->setMethod(Zend_Form::METHOD_POST)->setAction('plugin/quizz/run/showResults');

        $this->addElement('text', 'firstName', array(
                'id'          => 'quizz-firstname',
                'placeholder' => 'Name',
                'autocomplete' => 'off',
                'filters'     => array('StringTrim'),
                'validators'  => array(array('StringLength', true, array(1, 40)))
            ));

        $this->addElement('text', 'lastName', array(
                'id'          => 'quizz-lastname',
                'placeholder' => 'Last name',
                'autocomplete' => 'off',
                'filters'     => array('StringTrim'),
                'validators'  => array(array('StringLength', true, array(1, 40)))
            ));

        $this->addElement('text', 'email', array(
                'id'          => 'quizz-email',
                'placeholder' => 'Email',
                'autocomplete' => 'off',
                'filters'     => array('StringTrim'),
                'validators'  => array('EmailAddress')
            ));

        $this->addElement('text', 'phone', array(
                'id'           => 'quizz-phone',
                'placeholder'  => 'Phone (optional)',
                'autocomplete' => 'off',
                'filters'      => array('StringTrim'),
                'validators'   => array(array('StringLength', true, array(1, 40)))
            ));

        $this->addElement('hidden', 'answersKey', array(
                'filters'     => array('StringTrim'),
            ));

        $this->addElement('hidden', 'quizzId', array(
                'validators'  => array(new Zend_Validate_Int()),
            ));

        $this->addElement('submit', 'submit', array(
                'id'     => 'quizz-submit',
                'class'  => 'formsubmit',
                'label'  => 'Get My Results'
            ));

        $this->setElementDecorators(array('ViewHelper'));
    }

    static public function defaultFormElements(){

        return array('firstName', 'lastName', 'email', 'phone');
    }
}