<?php

/**
 * Class Quizz_Models_Mappers_QuizMapper
 */

class Quizz_Models_Mappers_QuizMapper extends Application_Model_Mappers_Abstract {

    protected $_dbTable = 'Quizz_Models_DbTables_QuizDbTable';

    protected $_model   = 'Quizz_Models_Model_Quiz';

    /**
     * Save the Quizz_Models_Model_Quiz to the database
     *
     * @param Quizz_Models_Model_Quiz $model
     * @return mixed
     * @throws Exceptions_QuizzException
     */
    public function save($model) {
        if(!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $dataQuiz   = array(
            'pageId'     => $model->getPageId(),
            'quizName'   => $model->getQuizName(),
            'quizTitle'  => $model->getQuizTitle(),
            'published'  => $model->getPublished(),
            'modifyDate' => $model->getModifyDate(),
        );
        $dataQuestions = $model->getQuestions();
        $dataAnswers = $model->getAnswers();
        $whereQuiz     = $this->getDbTable()->getAdapter()->quoteInto('pageId = ?', $dataQuiz['pageId']);
        $whereQuiz     .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('quizName = ?', $dataQuiz['quizName'] );
        $oldQuiz = $this->fetchAll($whereQuiz);
        if(empty($oldQuiz)){
            $quizId = $this->getDbTable()->insert($dataQuiz);
            if(!empty($dataQuestions)){
                $questionIds = $this->_insertQuestions($quizId, $dataQuestions);
                if(!empty($dataAnswers)){
                    $this->_insertAnswers($questionIds, $dataAnswers);
                }
            }
            return $quizId;
        } else {
            $id = $oldQuiz[0]->getId();
            $this->getDbTable()->update($dataQuiz, array('id = ?' => $id));
            //insert new questions and answers
            if(!empty($dataQuestions)){
                //delete question and insert new
                $questionId = $this->_insertQuestions($id, $dataQuestions);
                $this->_deleteAnswers($questionId);
                if(!empty($dataAnswers)){
                    $this->_insertAnswers($questionId, $dataAnswers);
                }
            }
            return $id;
        }
    }

    /**
     * @param $quizId
     * @param array $conditions
     */
    public function saveConditions($quizId, $conditions = array()){
        if(!empty($conditions)){
            $conditionsDbTable = new Quizz_Models_DbTables_ConditionsDbTable();
            foreach($conditions as $condType => $condValue){
                $where = $conditionsDbTable->getAdapter()->quoteInto('quizId = ?', $quizId);
                $where .= ' AND ' .$conditionsDbTable->getAdapter()->quoteInto('condtype = ?', $condType);
                $conditionsDbTable->getAdapter()->delete('plugin_quizz_conditions', $where);
                $conditionsData = array(
                    'quizId'    =>  $quizId,
                    'condtype'  =>  $condType,
                    'points'    =>  intval($condValue)
                );
                $conditionsDbTable->insert($conditionsData);
             }
        }
    }

    /**
     * insert questions into questions table
     * @param $quizId integer quiz id
     * @param $dataQuestions array of questions
     * @return array of ids of newly inserted questions
     */
    protected function _insertQuestions($quizId, $dataQuestions){
        $questionsDbTable = new Quizz_Models_DbTables_QuestionsDbTable();
        foreach($dataQuestions as $key => $dataInfo){
            $this->deleteQuestion($quizId, $key);
            $dataQuestions = array(
                'quizId'   => $quizId,
                'position' => $key,
                'question' => trim($dataQuestions[$key]['question']),
                'points'   => intval($dataQuestions[$key]['point']),
                'published'=> intval($dataQuestions[$key]['published'])
            );
        }
        return $questionsDbTable->insert($dataQuestions);
    }

    /**
     * @param $questionId
     * @param $dataAnswers
     * @return int|null
     */
    protected function _insertAnswers($questionId, $dataAnswers){
        $answersDbTable = new Quizz_Models_DbTables_AnswersDbTable();
        $sql = "INSERT INTO `plugin_quizz_answers` (`questionId`, `weight`, `answer`, `status`, `type`) VALUES ";
        try {
            foreach($dataAnswers as $weight => $answer){
                    $sql .= "(" . $questionId . ", " .$weight . ", '" . $answer['answer'] . "', " . $answer['status'] . ", '" . $answer['type'] . "')";
                    $sql .= ", ";
            }
            $sql = rtrim($sql, ", ");
            $stmt = $answersDbTable->getAdapter()->prepare($sql);
            $stmt->execute();
        } catch (Exception $e) {
            $errMess = $e->getMessage();
            error_log('error insertingAnswers: '. $errMess ."\n" . "sql = " . $sql);
            return null;
        }
        return $stmt->rowCount();
    }

    /**
     * @param $quizId
     * @param $position
     */
    public function deleteQuestion($quizId, $position){
        $questionsDbTable = new Quizz_Models_DbTables_QuestionsDbTable();
        $where = $questionsDbTable->getAdapter()->quoteInto('quizId = ?', $quizId);
        $where .= ' AND ' .$questionsDbTable->getAdapter()->quoteInto('position = ?', $position);
        $questionsDbTable->getAdapter()->delete('plugin_quizz_questions', $where);
    }

    /**
     * @param $questionId
     */
    protected function _deleteAnswers($questionId){
       $answersDbTable = new Quizz_Models_DbTables_AnswersDbTable();
       $where = $answersDbTable->getAdapter()->quoteInto('questionId = ?', $questionId);
       $answersDbTable->getAdapter()->delete('plugin_quizz_answers', $where);
    }

    /**
     * @param $quizName
     * @param $pageId
     * @return array
     */
    public function getQuizByNamePageId($quizName, $pageId){
        $whereQuiz  = $this->getDbTable()->getAdapter()->quoteInto('pageId = ?', $pageId);
        $whereQuiz .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('quizName = ?', $quizName);
        $select = $this->getDbTable()->select()->setIntegrityCheck(false)
            ->from(array('qu' => 'plugin_quizz_quiz'), array('quizId' => 'id', 'quizName', 'quizTitle'))
            ->joinLeft(array('qw' => 'plugin_quizz_questions'), 'qw.quizId = qu.id', array('questid' => 'id','position','question','points'))
            ->joinLeft(array('an' => 'plugin_quizz_answers'), 'an.questionId = qw.id', array('questid' => 'questionId', 'weight', 'answer', 'status'))
            ->where($whereQuiz)->order('qw.position');
        $resultSet = $this->getDbTable()->getAdapter()->fetchAll($select);
        if(!empty($resultSet)){
            return $this->_formatQuiz($resultSet);
        }
        return $resultSet;
    }

    /**
     * @param $quizName
     * @param $pageId
     * @return array|null
     */
    public function checkExistingQuiz($quizName, $pageId){
        $whereQuiz     = $this->getDbTable()->getAdapter()->quoteInto('pageId = ?', $pageId);
        $whereQuiz     .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('quizName = ?', $quizName);
        return $this->fetchAll($whereQuiz);
    }

    /**
     * @param $quiz
     * @return array
     */
    private function _formatQuiz($quiz){
        $data = array();
        $data['quizName'] = $quiz[0]['quizName'];
        $data['quizTitle'] = $quiz[0]['quizTitle'];
        $data['quizId'] = $quiz[0]['quizId'];
        $currQuestId = '';
        $questions = array();
        $answers = array();
        $ans = 0;
        $qnum = 1;
        foreach($quiz as $key => $value){
            if($currQuestId != $value['questid']){
                $ans = 1;
                if($key != 0){
                    $questions['answers'] = $answers;
                    $answers = '';
                    $data['questions'][$qnum] = $questions;
                    $questions = '';
                    $qnum++;
                }
                $currQuestId = $value['questid'];
                $questions['id'] = $value['questid'];
                $questions['position'] = $value['position'];
                $questions['question'] = $value['question'];
                $questions['points'] = $value['points'];
                $answers[$ans]['questid'] =  $value['questid'];
                $answers[$ans]['weight'] =  $value['weight'];
                $answers[$ans]['answer'] =  $value['answer'];
                $answers[$ans]['status'] =  $value['status'];
                $ans++;

            }else{
                $answers[$ans]['questid'] =  $value['questid'];
                $answers[$ans]['weight'] =  $value['weight'];
                $answers[$ans]['answer'] =  $value['answer'];
                $answers[$ans]['status'] =  $value['status'];
                $ans++;
            }
        }
        $questions['answers'] = $answers;
        $data['questions'][$qnum] = $questions;
        return $data;
    }

    /**
     * get quiz conditions to evaluate customer result
     * @param $id
     * @return Zend_Db_Table_Rowset_Abstract
     */
    public function getQuizConditions($id){
        $conditionsDbTable = new Quizz_Models_DbTables_ConditionsDbTable();
        $select = $this->getDbTable()->select()->setIntegrityCheck(false)
            ->from(array('qu' => 'plugin_quizz_conditions'), array('condtype', 'points'))
            ->where('quizId = ?', $id);
        return $conditionsDbTable->getAdapter()->fetchAssoc($select);
    }

    /**
     * get correct answers
     * @param $id
     */
    public function getCorrectAnswers($id){
       $select = $this->getDbTable()->select()->setIntegrityCheck(false)
           ->from(array('qu' => 'plugin_quizz_quiz'), null)
           ->joinLeft(array('qw' => 'plugin_quizz_questions'), 'qw.quizId = qu.id', 'id')
           ->joinLeft(array('an' => 'plugin_quizz_answers'), 'an.questionId = qw.id', 'weight')
           ->where('`qu`.`id` = ?', $id)
           ->where('`an`.`status` = ?', "1");
       return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * @param $id
     * @return array
     */
    public function calculateQuizResult($id){
        $questionIds = array_values($id);
        $where = $this->getDbTable()->getAdapter()->quoteInto('id IN(?)', $questionIds);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_quizz_questions', array('pointTotal' => 'SUM(points)'))
            ->where($where)->group('quizId');
        $result = $this->getDbTable()->getAdapter()->fetchAll($select);
        return $result[0]['pointTotal'];
    }

    /**
     * @param $quizName
     * @param $quizId
     */
    public function deleteQuiz($quizName, $pageId){
        $where = $this->getDbTable()->getAdapter()->quoteInto('quizName = ?', $quizName);
        $where .= ' AND ' .$this->getDbTable()->getAdapter()->quoteInto('pageId = ?', $pageId);
        $this->getDbTable()->delete($where);
    }

    /**
     * @param $quizId
     * @return mixed
     */
    public function getQuizTotalScore($quizId){
        $where = $this->getDbTable()->getAdapter()->quoteInto('quizId = ?', $quizId);
        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_quizz_questions', array('pointTotal' => 'SUM(points)'))
            ->where($where)->group('quizId');
        $result = $this->getDbTable()->getAdapter()->fetchAll($select);
        return $result[0]['pointTotal'];
    }
}
