<?php

/**
 * send quiz result to the user with action email
 * Class Quizz_Tools_MailWatchdog
 *
 */
class Quizz_Tools_MailWatchdog implements Interfaces_Observer {


    const TRIGGER_QUIZ_PASSED     = 'quiz_passed';

    private $_options = array();

    /**
     * @var Tools_Mail_Mailer instance of mailer
     */
    private $_mailer;

    /**
     * @var Helpers_Action_Config
     */
    private $_configHelper;

    /**
     * @var Helpers_Action_Website
     */
    private $_websiteHelper;

    /**
     * @var Instance of watched object
     */
    private $_object;

    private $_template;

    /**
     * @var result
     */
    private $_result;

    public function __construct($options = array()) {

        $this->_configHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('config');
        $this->_websiteHelper = Zend_Controller_Action_HelperBroker::getExistingHelper('website');
        $this->_options = $options;
        $this->_initMailer();
        $this->_entityParser = new Tools_Content_EntityParser();
    }

    /**
     *  set mailer
     */
    private function _initMailer(){
        $config = $this->_configHelper->getConfig();
        $this->_mailer = new Tools_Mail_Mailer();

        if ((bool)$config['useSmtp']){
            $smtpConfig = array(
                'host'      => $config['smtpHost'],
                'username'  => $config['smtpLogin'],
                'password'  => $config['smtpPassword']
            );
            if ((bool)$config['smtpSsl']){
                $smtpConfig['ssl'] = $config['smtpSsl'];
            }
            if (!empty($config['smtpPort'])){
                $smtpConfig['port'] = $config['smtpPort'];
            }
            $this->_mailer->setSmtpConfig($smtpConfig);
            $this->_mailer->setTransport(Tools_Mail_Mailer::MAIL_TYPE_SMTP);
        } else {
            $this->_mailer->setTransport(Tools_Mail_Mailer::MAIL_TYPE_MAIL);
        }
    }

    /**
     * @param $object
     * @return bool
     * @throws Exceptions_SeotoasterException
     */
    public function notify($object) {
        if (!$object){
            return false;
        }

        $this->_object = $object;

        if (isset($this->_options['template']) && !empty($this->_options['template']) ){
            $this->_template = $this->_preparseEmailTemplate();
        } else {
            throw new Exceptions_SeotoasterException('Missing template for action email trigger');
        }
        $this->_subject = $this->_options['subject'];

        if(!empty($this->_options['from'])){
            preg_match('/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})/i', $this->_options['from'], $matches);
            $fromMail  = current($matches);
            $labelText = preg_replace('/([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)/','',$this->_options['from']);
            $this->_mailer->setMailFromLabel($labelText);
        } else {
            $this->_mailer->setMailFromLabel($this->_storeConfig['company']);
        }

        if (!empty($this->_options['from']) && !empty($fromMail)){
            $this->_mailer->setMailFrom($fromMail);
        } else {
            $this->_mailer->setMailFrom($this->_configHelper->getConfig('adminEmail'));
        }

        if (isset($this->_options['trigger'])){
            $methodName = str_replace('quiz_', '', $this->_options['trigger']);
            $methodName = '_send'.ucfirst(strtolower(preg_replace('/\s*/', '', $methodName))).'Mail';
            if (method_exists($this, $methodName)){
                $this->$methodName();
            }
        }
    }

    /**
     * @return bool
     * @throws Exceptions_SeotoasterException
     */
    protected function _sendPassedMail(){
        $systemConfig = $this->_configHelper->getConfig();
        $userMapper = Application_Model_Mappers_UserMapper::getInstance();
        $adminBccArray = array();
        $configFormMapper = Quizz_Models_Mappers_CFMapper::getInstance();
        $configData = unserialize($configFormMapper->getFormDataByQuizID($this->_options['quizId']));
        $adminEmail = !empty($configData['email_sender']) ? $configData['email_sender']: $systemConfig['adminEmail'] ;
        switch ($this->_options['recipient']) {
            case Tools_Security_Acl::ROLE_ADMIN:
                $this->_mailer->setMailToLabel('Quiz Result')
                    ->setMailTo($adminEmail);
                $where = $userMapper->getDbTable()->getAdapter()->quoteInto("role_id = ?", Tools_Security_Acl::ROLE_ADMIN);
                $adminUsers = $userMapper->fetchAll($where);
                if(!empty($adminUsers)){
                    foreach($adminUsers as $admin){
                        array_push($adminBccArray, $admin->getEmail());
                    }
                    if(!empty($adminBccArray)){
                        $this->_mailer->setMailBcc($adminBccArray);
                    }
                }
                break;
            case Tools_Security_Acl::ROLE_GUEST:
                $this->_mailer->setMailToLabel($this->_options['userName'])->setMailTo($this->_options['mailTo']);
                break;
            default:
                error_log('Unsupported recipient '.$this->_options['recipient'].' given');
                return false;
                break;
        }

        $this->_entityParser->objectToDictionary($this->_object);
        $this->_entityParser->addToDictionary(array('quiz:title'=>$this->_options['title'], 'quiz:score'=>$this->_options['score'],
                                            'quiz:email'=>$this->_options['mailTo'], 'quiz:name'=>$this->_options['userName'],
                                            'quiz:phone'=>$this->_options['phone'], 'quiz:date'=>$this->_options['date'],
                                            'quiz:containertext' => $this->_options['custom_mail_body']
                                            ));
        return $this->_send();
    }

    /**
     * @return bool|null
     */
    private function _preparseEmailTemplate(){
        $tmplName = $this->_options['template'];
        $tmplMessage = $this->_options['message'];
        $mailTemplate = Application_Model_Mappers_TemplateMapper::getInstance()->find($tmplName);

        if (!empty($mailTemplate)){
            $this->_entityParser->setDictionary(array(
                    'emailmessage' => !empty($tmplMessage) ? nl2br($tmplMessage) : ''
                ));
            //pushing message template to email template and cleaning dictionary
            $mailTemplate = $this->_entityParser->parse($mailTemplate->getContent());
            $this->_entityParser->setDictionary(array());

            $mailTemplate = $this->_entityParser->parse($mailTemplate);

            $themeData = Zend_Registry::get('theme');
            $extConfig = Zend_Registry::get('extConfig');
            $parserOptions = array(
                'websiteUrl'   => $this->_websiteHelper->getUrl(),
                'websitePath'  => $this->_websiteHelper->getPath(),
                'currentTheme' => $extConfig['currentTheme'],
                'themePath'    => $themeData['path'],
            );
            $parser = new Tools_Content_Parser($mailTemplate, array(), $parserOptions);

            return $parser->parseSimple();
        }

        return false;
    }

    /**
     * @return bool
     * @throws Exceptions_SeotoasterException
     */
    protected function _send(){
        if (!$this->_mailer->getMailFrom() || !$this->_mailer->getMailTo()) {
            throw new Exceptions_SeotoasterException('Missing required "from" and "to" fields');
        }
        $this->_mailer->setSubject($this->_entityParser->parse($this->_subject));
        $this->_mailer->setBody($this->_entityParser->parse($this->_template));
        return ($this->_mailer->send() !== false);
    }
}